<?php

namespace Lean;

use Notes\Annotation;
use Taxus\Taxus;

use League\Route\RouteGroup,
    League\Route\Router;

use Psr\Http\Message\ServerRequestInterface,
    Psr\Http\Message\ResponseInterface,
    Psr\Container\ContainerInterface;

use Notes\Route\RouteFetcher;

use Notes\Security\SecurityHandler;

use Notes\Tell\LanguageHandler;

use Picea\Picea,
    Picea\Extension\UrlExtension;

use Storage\Cookie,
    Storage\Session;

use Mcnd\Event\EventManager;

use function DI\autowire, DI\create;

class Routing {

    public Annotation $selectedRoute;

    public ResponseInterface $response;

    public function __construct(
        public Session $session,
        public Cookie $cookie,
        public UrlExtension $extension,
        public Router $router,
        public RouteFetcher $fetcher,
        public SecurityHandler $security,
        public LanguageHandler $language,
        public Taxus $taxus,
        public EventManager $eventManager,
    ) { }

    public function registerRoute(ContainerInterface $container, string $urlBase) {
        $this->router->group(rtrim($urlBase, "/"), function (RouteGroup $route) use ($container) {
            foreach($this->fetcher->compile() as $annotation) {
                $this->eventManager->execute(Event\RoutingCompileRoutes::class, $this, $annotation);

                /* @deprecated annotation->method will become standard when using native attributes */
                foreach((array) ( $annotation->method ?? $annotation->methods ) as $method) {
                    # Mapping every URLs from annotations in searched folders (Api, Controller, etc...)
                    $route->map(strtoupper($method), $annotation->getRoute(), function (ServerRequestInterface $request, array $arguments) use (
                        $container, $route, $annotation
                    ) : ResponseInterface
                    {
                        $class = $annotation->class;    
                        $method = $annotation->classMethod;
                        $object = $container->get($class);

                        $this->eventManager->execute(Event\RoutingMapRoutes::class, $this, $container, $request, $annotation);

                        $container->set(ServerRequestInterface::class, $request);

                        return $this->response ?? $object->$method($request, $arguments);
                    });
                }
            }
        });
    }
}
